# Generated by Django 2.2.28 on 2025-10-10 09:25

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_vr_payment', '0002_add_webhooks'),
    ]

    operations = [
        migrations.AlterField(
            model_name='vrpaymentbasicpayment',
            name='descriptor',
            field=models.CharField(blank=True, help_text="Can be used to populate all or part of the Merchant Name descriptor, which often appears on the first line of the shopper's statement. The full use of this field depends on the Merchant Account configuration. NOTE: merchant.name can override any data sent in this field.", max_length=255, null=True, verbose_name='Descriptor'),
        ),
        migrations.AlterField(
            model_name='vrpaymentbasicpaymentstatusresponse',
            name='descriptor',
            field=models.CharField(blank=True, help_text='The descriptor of the request.', max_length=255, null=True, verbose_name='Descriptor'),
        ),
    ]
